/* cas concret du chapitre 7 */
/* dfinition de la bibliothque de travail */
LIBNAME banque "c:\sas\exemple banque\" ;

/* nom des objets crs par la procdure CONTENTS */
ODS TRACE ON ;
  PROC CONTENTS DATA = banque.cc ;
  RUN ;
ODS TRACE OFF ;

/* constitution du dictionnaire des donnes */
ODS RTF FILE = "c:\sas\dictionnaire exemple banque.doc" ;
ODS RTF SELECT Attributes VariablesAlpha ;
PROC CONTENTS DATA = banque._ALL_ ;
RUN ;
ODS RTF CLOSE ;

/* importation de la mise  jour des donnes : 1re version */
DATA banque.operation_cc_mois_courant ;
   INFILE "C:\sas\reu\txt_operation_cc.dat" ;
   INPUT @1   id_cc                $
         @11  date_op              
         @20  cd_nature_op         $
         @21  rang_op              
         @23  mt_op                
         @35  txt_debiteur_op      $
         @235 txt_beneficiaire_op  $
         @435 b_validation_op      
         @437 date_effet_op        
   ;
RUN ;

/* importation de la mise  jour des donnes : 2e version */
DATA banque.operation_cc_mois_courant ;
   INFILE "C:\sas\reu\txt_operation_cc.dat" ;
   INPUT @1   id_cc                $10.
         @11  date_op              YYMMDD8.
         @20  cd_nature_op         $1.
         @21  rang_op              2.
         @23  mt_op                NUMX10.2
         @35  txt_debiteur_op      $200.
         @235 txt_beneficiaire_op  $200.
         @435 b_validation_op      1.
         @437 date_effet_op        DDMMYY10.
   ;
RUN ;

/* importation de la mise  jour des donnes : 3e version */
DATA banque.operation_cc_mois_courant ;
   INFILE "C:\sas\reu\txt_operation_cc.dat" LRECL = 500 MISSOVER ;
   INPUT @1   id_cc                $10.
         @11  date_op              YYMMDD8.
         @20  cd_nature_op         $1.
         @21  rang_op              2.
         @23  mt_op                NUMX10.2
         @35  txt_debiteur_op      $200.
         @235 txt_beneficiaire_op  $200.
         @435 b_validation_op      1.
         @437 date_effet_op        DDMMYY10.
   ;
RUN ;

/* importation de la mise  jour des donnes : 4e et dernire version */
DATA banque.operation_cc_mois_courant ;
   ATTRIB 
     ID_CC 
       LABEL = "N de compte courant"
     DATE_OP
       LABEL = "Date de l'opration"
       FORMAT = DDMMYY10.
       LENGTH = 4
     CD_NATURE_OP
       LABEL = "Nature de l'opration : R=retrait, V=virement,  
                                        P=paiement CB, C=chque"
     RANG_OP
       LABEL = "Rang de l'opration dans la journe"
       LENGTH = 3
     MT_OP
       LABEL = "Montant de l'opration : positif = crdit, 
                                         ngatif = dbit"
     TXT_DEBITEUR_OP 
       LABEL = "Dbiteur de l'oprateur"
     TXT_BENEFICIAIRE_OP
       LABEL = "Crditeur de l'opration"
     B_VALIDATION_OP 
       LABEL = "Opration valide ?"
       LENGTH = 3
     DATE_EFFET_OP 
       LABEL = "Date d'effet de l'opration"
       FORMAT = DDMMYY10.
       LENGTH = 4
   ;  
   INFILE "C:\sas\reu\txt_operation_cc.dat" LRECL = 500 MISSOVER ;
   INPUT @1   id_cc                $10.
         @11  date_op              YYMMDD8.
         @20  cd_nature_op         $1.
         @21  rang_op              2.
         @23  mt_op                NUMX10.2
         @35  txt_debiteur_op      $200.
         @235 txt_beneficiaire_op  $200.
         @435 b_validation_op      1.
         @437 date_effet_op        DDMMYY10.
   ;
RUN ;

/* mise  jour des transactions bancaires */
DATA banque.operation_cc ;
  SET banque.operation_cc
      banque.operation_cc_mois_courant
  ;
RUN ;

/* mise  jour des transactions bancaires avec historique glissant */
DATA banque.operation_cc ;
  SET banque.operation_cc (WHERE = (date_op >= INTNX("MONTH",
                                                     TODAY(),
                                                     -12
                                                    )
                                   )
                          )
      banque.operation_cc_mois_courant
  ;
RUN ;

/* mise  jour des statistiques, tape 1 : cration de boolens */
DATA work.op_cc_validees ;
  SET banque.operation_cc_mois_courant (WHERE=(b_validation_op = 1));
  IF mt_op > 0 THEN tmp_mvt_credit = 1 ;
  ELSE tmp_mvt_credit = 0 ;
  IF mt_op < 0 THEN tmp_mvt_debit  = 1 ;
  ELSE tmp_mvt_debit = 0 ;
  IF cd_nature_op = "C" AND mt_op < 0 THEN tmp_ch_emis = 1 ;
  ELSE tmp_ch_emis = 0 ;
  IF cd_nature_op = "C" AND mt_op > 0 THEN tmp_ch_recu = 1 ;
  ELSE tmp_ch_recus = 0 ;
  IF cd_nature_op = "P" AND mt_op < 0 THEN tmp_pay_cb = 1 ;
  ELSE tmp_pay_cb = 0 ;
  IF cd_nature_op = "R" AND mt_op < 0 THEN tmp_ret_cb = 1 ;
  ELSE tmp_ret_cb = 0 ;
RUN ;

/* mise  jour des statistiques, tape 2 : calculs */
ODS OUTPUT summary = work.resume1_cc (DROP = nobs) ;
  PROC MEANS DATA = work.op_cc_validees MAX ;
    VAR date_op ;
    CLASS id_cc ;
  RUN ;
ODS OUTPUT CLOSE ;
ODS OUTPUT summary = work.resume2_cc (DROP = vname: label: nobs
                 RENAME = (tmp_mvt_credit_sum = nb_mvt_crediteurs
                           tmp_mvt_debit_sum  = nb_mvt_debiteurs
                           tmp_ch_emis_sum    = nb_chq_emis
                           tmp_ch_recu_sum    = nb_chq_recus
                           tmp_pay_cb_sum     = nb_pay_cb
                           tmp_ret_cb_sum     = nb_ret_cb
                           mt_op_sum          = total_operations
	                     )) ;
PROC MEANS DATA = work.op_cc_validees SUM ;
	VAR tmp_: mt_op ;
	CLASS id_cc ;
RUN ;

/* mise  jour des statistiques, tape 3 : jointures */
PROC SORT DATA = work.resume1_cc ;
	BY id_cc ;
RUN ;
PROC SORT DATA = work.resume2_cc ;
	BY id_cc ;
RUN ;
PROC SORT DATA = banque.cc ;
	BY id_cc ;
RUN ;
DATA banque.cc (DROP = total_operations) ;
  MERGE banque.cc
        work.resume1_cc (IN = a_fait_au_moins_une_operation)
        work.resume2_cc
  ;
  BY id_cc ;
  cd_mois = PUT(INTNX("MONTH",TODAY(),-1), MMYYS7.) ;
  date_derniere_op = MAX(date_derniere_op,date_op_max) ;
  solde_fin_mois_precedent = solde_fin_mois ;
  IF a_fait_au_moins_une_operation THEN 
     solde_fin_mois = solde_fin_mois_precedent + total_operations ;
  ELSE solde_fin_mois = solde_fin_mois_precedent ;
RUN ;

/* nombre de chques mis : version 1 */
PROC GCHART DATA = banque.cc ;
  VBAR3D nb_chq_emis ;
RUN ; QUIT ;

/* nombre de chques mis : version 2 */
PROC GCHART DATA = banque.cc ;
  VBAR3D nb_chq_emis / DISCRETE ;
RUN ; QUIT ;

/* nombre de chques mis : version 3 */
ODS OUTPUT summary = work.stats (KEEP = nb_chq_emis_sum) ;
PROC MEANS DATA = banque.cc SUM ;
	VAR nb_chq_emis ;
RUN ;
DATA _NULL_ ;
	SET work.stats ;
	CALL SYMPUT ("total_cheques",nb_chq_emis_sum) ;
RUN ;
TITLE1 "Nombre total de chques mis en mars : &total_cheques" ;
PROC GCHART DATA = banque.cc ;
	VBAR3D nb_chq_emis / DISCRETE ;
RUN ; QUIT ;

/* utilisation de la carte bancaire, tape 1 : prparation des donnes */
PROC SORT DATA = banque.operation_cc 
                           (KEEP = mt_op id_cc date_op 
                                   b_validation_op cd_nature_op
                           WHERE = (date_op BETWEEN "01MAR2004"d AND
                                                    "31MAR2004"d AND
                                    b_validation_op = 1 AND
                                    cd_nature_op = "P")
                           )
       OUT = work.op_cc (KEEP = mt_op id_cc) ;
  BY id_cc ;
RUN ;
PROC SORT DATA = banque.cc (KEEP = id_contrat id_cc)
           OUT = work.cc ;
  BY id_cc ;
RUN ;
DATA work.tableau ; /* jointure de OPERATION_CC et de CC */
  MERGE work.op_cc (IN = a_fait_un_paiement)
        work.cc
  ;
  BY id_cc ;
  IF a_fait_un_paiement THEN OUTPUT ;
RUN ;
PROC SORT DATA = work.tableau ; 
  BY id_contrat ;
RUN ;
PROC SORT DATA = banque.contrat (KEEP = id_contrat id_pers)
           OUT = work.cont ;
  BY id_contrat ;
RUN ;
DATA work.tableau ; /* jointure du prcdent rsultat avec CONTRAT */
  MERGE work.tableau (IN = a_un_CC)
        work.cont
  ;
  BY id_contrat ;
  IF a_un_CC THEN OUTPUT ;
RUN ;
PROC SORT DATA = work.tableau (KEEP = id_pers mt_op) ;
  BY id_pers ;
RUN ;
PROC SORT DATA = banque.client (KEEP = id_pers cd_segment  
                                       id_agence_principale)
           OUT = work.cli ;
  BY id_pers ;
RUN ;
DATA work.tableau   /* jointure du prcdent rsultat avec CLIENT */
          (DROP = mt_op) ; 
  MERGE work.tableau (IN = a_un_CC)
      work.cli
  ;
  BY id_pers ;
  montant_CB = -1 * mt_op ;
  IF a_un_CC THEN OUTPUT ;
RUN ;

/* utilisation de la carte bancaire, tape 2 : premire version du tableau */
PROC TABULATE DATA = work.tableau ;
  CLASS cd_segment id_agence_principale ;
  VAR montant_CB ;
  TABLE (cd_segment)           ,
        (id_agence_principale) *
        (montant_CB)           *
        (MEAN) 
  ;
RUN ;

/* utilisation de la carte bancaire, tape 3 : version dfinitive du tableau */
PROC TABULATE DATA = work.tableau ;
  CLASS cd_segment id_agence_principale ;
  VAR montant_CB ;
  TABLE (cd_segment=""        ALL="Tous les segments" ) ,
        (id_agence_principale ALL="Toutes les agences") *
        (montant_CB = ""                              ) *
        (MEAN = "Montant moyen des paiements par CB"  ) *
         FORMAT = NUMX7.2
        / BOX = cd_segment
  ;
RUN ;

/* frquence de clients possdant plusieurs produits : constitution des donnes */
PROC SORT DATA = banque.cc (KEEP = id_contrat date_derniere_op
        WHERE = (date_derniere_op >= INTNX("MONTH",TODAY(),-1)))
       OUT = work.cc (KEEP = id_contrat) ;
  BY id_contrat ;
RUN ;
PROC SORT DATA = banque.contrat (KEEP = id_contrat id_pers) 
           OUT = work.cont ;
  BY id_contrat ;
RUN ;
DATA work.clients_cc (KEEP = id_pers) ;
  MERGE work.cc (IN = cc_actif)
        work.cont
  ;
  BY id_contrat ;
  IF cc_actif THEN OUTPUT ;
RUN ;
PROC SORT DATA = banque.livret_epargne (KEEP = id_contrat)
           OUT = work.cc ;
  BY id_contrat ;
RUN ;
DATA work.clients_livret (KEEP = id_pers) ;
  MERGE work.cc (IN = livret)
        work.cont
  ;
  BY id_contrat ;
  IF livret THEN OUTPUT ;
RUN ;
PROC SORT DATA = work.clients_cc NODUPKEY ;
  BY id_pers ;
RUN ;
PROC SORT DATA = work.clients_livret ;
  BY id_pers ;
RUN ;
DATA work.clients ;
  MERGE work.clients_cc     (IN = cc)
        work.clients_livret (IN = livret)
  ;
  BY id_pers ;
  IF cc     THEN possede_cc     = "OUI" ;
  ELSE           possede_cc     = "NON" ;
  IF livret THEN possede_livret = "OUI" ;
  ELSE           possede_livret = "NON" ;
RUN ;

/* frquence de clients possdant plusieurs produits : tableau final */
PROC FREQ DATA = work.clients ;
  TABLE possede_cc * possede_livret ;
RUN ;

/* informations clients en feuille Excel : version sans paramtrage */
PROC SORT DATA = banque.operation_cc ;
  BY id_cc date_effet_op ;
RUN ;
DATA work.temp ;
  SET banque.operation_cc 
         (WHERE = (date_op >= INTNX("MONTH",TODAY(),-1)
               AND cd_nature_op = "R" AND id_cc = "000000104")) ;
RUN ;
PROC EXPORT DATA = work.temp
      OUTFILE = "c:\temp\operations.xls"
      DBMS = EXCEL
      REPLACE ;
  SHEET = Retraits ;
RUN ;

/* informations clients en feuille Excel : version n 1 avec paramtrage */
%LET mv_op = C ;
%LET mv_cc = 000001152 ;
/*******************************************************************/
PROC SORT DATA = banque.operation_cc ;
  BY id_cc date_effet_op ;
RUN ;
PROC FORMAT ;
	VALUE $nat_op
		"R" = "Retraits"
		"V" = "Virements"
		"P" = "CB"
		"C" = "Chques"
	;
RUN ;
DATA work.temp ;
  SET banque.operation_cc 
         (WHERE = (date_op >= INTNX("MONTH",TODAY(),-1)
               AND cd_nature_op = "&mv_op" AND id_cc = "&mv_cc")) ;
RUN ;
PROC EXPORT DATA = work.temp
      OUTFILE = "c:\temp\operations.xls"
      DBMS = EXCEL
      REPLACE ;
  SHEET = %SYSFUNC (LEFT(&mv_op),$nat_op.) ;
RUN ;

/* informations clients en feuille Excel : version n 2 avec paramtrage */
/* collecte d'informations pour l'automatisation */
PROC SORT DATA = banque.operation_cc 
              (WHERE = (date_op >= INTNX("MONTH",TODAY(),-1)
                    AND id_cc = "&mv_cc"))
           OUT = work.op (KEEP = cd_nature_op)
           NODUPKEY ;
  BY cd_nature_op ;
RUN ;
DATA _NULL_ ;
  SET work.op ;
  CALL SYMPUT(COMPRESS("mv_op"!!_N_),cd_nature_op) ;
  CALL SYMPUT("nb_op",_N_) ;
RUN ;
%PUT _USER_ ;

/* macro-programme dfinitif */
%MACRO edition_op (mv_cc) ;
  PROC SORT DATA = banque.operation_cc 
                     (WHERE = (date_op >= INTNX("MONTH",TODAY(),-1)
                           AND id_cc = "&mv_cc"))
         OUT = work.temp ;
    BY id_cc date_effet_op ;
  RUN ;
  PROC FORMAT ;
    VALUE $nat_op
      "R" = "Retraits"
      "V" = "Virements"
      "P" = "CB"
      "C" = "Chques"
    ;
  RUN ;
  PROC SORT DATA = banque.operation_cc 
           (WHERE = (date_op >= INTNX("MONTH",TODAY(),-1)
                 AND id_cc = "&mv_cc"))
         OUT = work.op (KEEP = cd_nature_op)
            NODUPKEY ;
    BY cd_nature_op ;
  RUN ;
  DATA _NULL_ ;
    SET work.op ;
    CALL SYMPUT(COMPRESS("mv_op"!!_N_),cd_nature_op) ;
    CALL SYMPUT("nb_op",_N_) ;
  RUN ;
  %DO i = 1 %TO &nb_op ;
    DATA work.temp_op ;
      SET work.temp (WHERE = (cd_nature_op = "&&mv_op&i")) ;
    RUN ;
    PROC EXPORT DATA = work.temp_op
          OUTFILE = "c:\temp\operations.xls"
          DBMS = EXCEL
          REPLACE ;
      SHEET = %SYSFUNC(LEFT(&&mv_op&i),$nat_op.) ;
    RUN ;
  %END ;
%MEND edition_op ;

/* utilisation du macro-programme */
%edition_op (000002277) ;
